﻿//=============================================================================
//  移動リサイズコントロール関係サンプル
//  楕円描画用のクラス 　GDI+ を使用
//
//=============================================================================
unit plGPResizeEllipse;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, plBaseDragResize,
  GDIPAPI, GDIPOBJ, GDIPUTIL;

type
  TplGPResizeEllipse = class(TplBaseDragResize)
  private
    FPenColor   : TGPColor;
    FPenWidth   : Single;
    FBrushColor : TGPColor;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DrawRoutine(ACanvas: TCanvas; AGPBitmap: TGPBitmap = nil;
      AOffsetX: Integer = 0; AOffsetY: Integer = 0); override;
  end;

implementation

uses
  Types,
  Math;

{ TplGPResizeEllipse }

//=============================================================================
//  コンストラクタ
//=============================================================================
constructor TplGPResizeEllipse.Create(AOwner: TComponent);
begin
  inherited;

  FBrushColor := aclTransparent;
  FPenColor := $FF00A0FF;
  FPenWidth := 8;
end;

//=============================================================================
//  デコンストラクタ処理
//=============================================================================
destructor TplGPResizeEllipse.Destroy;
begin
//
  inherited;
end;

//-----------------------------------------------------------------------------
//  実際の描画処理を実行するDrawRoutineメソッド
//  引数のACanvasにこのメソッドの描画コードを実行する
//
//  描画開始位置は(DrawLeft, DrawTop)
//  幅と高さは DrawWidth, DrawHeight
//-----------------------------------------------------------------------------
procedure TplGPResizeEllipse.DrawRoutine(ACanvas: TCanvas; AGPBitmap: TGPBitmap;
  AOffsetX: Integer; AOffsetY: Integer);
var
  LGPGraphic : TGPGraphics;
　LGPPen     : TGPPen;
  LGPBrush   : TGPSolidBrush;
  LHalfW     : Single;
  LHalfH     : Single;
  LGPRect    : TGPRectF;
begin
  inherited;

  // 以下、実際の描画

  if (FPenColor = aclTransparent) and (FBrushColor = aclTransparent) then exit;

  SetBounds(Left, Top, Width, Height);
  //ペンのオブジェクトを生成
  LGPPen := TGPPen.Create(FPenColor, FPenWidth);
  //TGPSolidBrushを生成
  LGPBrush := TGPSolidBrush.Create(FBrushColor);
  try
    // 描画領域を指定
    LHalfW := Sign(DrawWidth) * FPenWidth / 2.0;
    LHalfH := Sign(DrawHeight) * FPenWidth / 2.0;
    LGPRect := MakeRect(DrawLeft + Lhalfw, DrawTop + LhalfH,
                        Drawwidth - LHalfW * 2.0, DrawHeight - LHalfH * 2.0);

    // TGPGraphicsオブジェクトを生成
    LGPGraphic := TGPGraphics.Create(ACanvas.Handle);

    if FBrushColor <> aclTransparent then begin
      // 内部を塗り潰す
      LGPGraphic.FillEllipse(LGPBrush, LGPRect);
    end;

    if FPenColor <> aclTransparent then begin
      // 周囲の枠線を描画
      // 線の描画関係の関数名はDrawXXXX
      LGPGraphic.DrawEllipse(LGPPen, LGPRect);
    end;
  finally
    FreeAndNil(LGPBrush);
    FreeAndNil(LGPPen);
    FreeAndNil(LGPGraphic);
  end;
end;

end.
